/*#######################################################################################
ARM Small Webserver 

Copyright (C) 2006 Ulrich Radig

#######################################################################################*/

#ifndef _STACK_H
 #define _STACK_H

#include "typedefs.h"
#include "enc28j60.h"
#include "typedefs.h"
#include "printf_P.h"
 
U08 mymac[6];
U08 myip[4];

//Prototype
void ARP_Reply (U08 *, U16 *);
void ICMP_Reply (U08 *,U16 *);
U16 checksum (U08 *,U16,U32);
void Make_IP_Header (U08 *);

#define ETHER_OFFSET			0x00

struct Ethernet_Header	{
	U08 EnetPacketDest[6];	//MAC Zieladresse 6 Byte
	U08 EnetPacketSrc[6];	//MAC Quelladresse 6 Byte
	U16 EnetPacketType;		//Nutzlast 0x0800=IP Datagramm;0x0806 = ARP
};

//Aufbau eines ARP Header
//	
//	2 BYTE Hardware Typ				|	2 BYTE Protokoll Typ	
//	1 BYTE Lnge Hardwareadresse	|	1 BYTE Lnge Protokolladresse
//	2 BYTE Operation
//	6 BYTE MAC Adresse Absender		|	4 BYTE IP Adresse Absender
//	6 BYTE MAC Adresse Empfnger	|	4 BYTE IP Adresse Empfnger	

#define ARP_OFFSET				0x0E

//ARP Layout
struct ARP_Header	{
		U16 ARP_HWType;		//Hardware Typ enthlt den Code fr Ethernet oder andere Link Layer
		U16 ARP_PRType;		//Protokoll Typ enthlt den Code fr IP o. anderes bertragungsprotokoll
		U08 ARP_HWLen;		//Lnge der Hardwareadresse enthlt 6 fr 6 Byte lange MAC Adressen
		U08 ARP_PRLen;		//Lnge der Protocolladresse enthlt 4 fr 4 Byte lange IP Adressen
		U16 ARP_Op;			//Enthlt Code der signalisiert ob es sich um eine Anfrage o. Antwort handelt
		U08 ARP_SHAddr[6];	//Enthlt die MAC Adresse des Anfragenden  
		U32 ARP_SIPAddr;    //Enthlt die IP Adresse des Absenders
		U08 ARP_THAddr[6];	//MAC Adresse Ziel, ist in diesem Fall 6 * 00,da die Adresse erst noch herausgefunden werden soll (ARP Request)
		U32 ARP_TIPAddr;    //IP Adresse enthlt die IP Adresse zu der die Kommunikation aufgebaut werden soll 
};

//Aufbau eines IP Datagramms (B=BIT)
//	
//4B Version	|4B Headergr.	|8B Tos	|16B Gesamtlnge in Bytes	
//16B Identifikation			|3B Schalter	|13B Fragmentierungsposition
//8B Time to Live	|8B Protokoll	|16B Header Prfsumme 
//32B IP Quelladresse
//32B IB Zieladresse
//
//*** Nutzlast ***

#define IP_OFFSET				0x0E

struct IP_Header	{
	U08	IP_Vers_Len;	//4 BIT Die Versionsnummer von IP, 
									//meistens also 4 + 4Bit Headergre 	
	U08	IP_Tos;			//Type of Service
	U16	IP_Pktlen;		//16 Bit Komplette Lng des IP Datagrams in Bytes
	U16	IP_Id;			//ID des Packet fr Fragmentierung oder 
									//Reassemblierung
	U16	IP_Frag_Offset;	//wird benutzt um ein fragmentiertes 
									//IP Packet wieder korrekt zusammenzusetzen
	U08	IP_ttl;			//8 Bit Time to Live die lebenszeit eines Paket
	U08	IP_Proto;		//Zeigt das hherschichtige Protokoll an 
									//(TCP, UDP, ICMP)
	U16	IP_Hdr_Cksum;	//Checksumme des IP Headers
	U32	IP_Srcaddr;		//32 Bit IP Quelladresse
	U32	IP_Destaddr;	//32 Bit IP Zieladresse
};

//Aufbau einer ICMP Nachricht
//	
//8 BIT Typ	|8 BIT Code	|16 BIT Prfsumme	
//Je nach Typ			|Nachricht unterschiedlich
//Testdaten
//
//8 BIT Typ = 0 Echo Reply oder 8 Echo request	

#define ICMP_OFFSET				0x22

//ICMP Header
struct ICMP_Header {
		U08     ICMP_Type;		//8 bit typ identifiziert Aufgabe der Nachricht 
										//0 = Echo Reply 8 = Echo Request
		U08     ICMP_Code;		//8 Bit Code enthlt Detailangaben zu bestimmten Typen
		U16     ICMP_Cksum;		//16 Bit Prfsumme enthlt die CRC Prfsumme
		U08     ICMP_Id;		//1 byte Identifizierung
		U16     ICMP_SeqNum;	//Sequenznummer
};

//IP Header Layout
#define	IP_VERS_LEN				IP_OFFSET+0x00	//4 BIT Die Versionsnummer von IP, 
												//meistens also 4 + 4Bit Headergre 	
#define	IP_PKTLEN				IP_OFFSET+0x02	//16 Bit Komplette Lng des IP Datagrams in Bytes
#define	IP_SRCADDR				IP_OFFSET+0x0C	//32 Bit IP Quelladresse	

//IP Protocol Types
#define	PROT_ICMP				0x01	//zeigt an die Nutzlasten enthalten das ICMP Prot
#define	PROT_TCP				0x06	//zeigt an die Nutzlasten enthalten das TCP Prot.
#define	PROT_UDP				0x11	//zeigt an die Nutzlasten enthalten das UDP Prot.

//Defines fr IF Abfrage
#define MyIP 			ip->IP_Destaddr==*((U32*)&myip[0])
#define IP_UDP_Packet 	ip->IP_Proto == PROT_UDP
#define IP_TCP_Packet 	ip->IP_Proto == PROT_TCP
#define IP_ICMP_Packet 	ip->IP_Proto == PROT_ICMP

//Defines fr if Abfrage
#define Ethernet_IP_Datagramm	ethernet->EnetPacketType == 0x0008
#define Ethernet_ARP_Datagramm 	ethernet->EnetPacketType == 0x0608

#endif // #define _STACK_H
